@@ -53,20 +53,19 @@ static unsigned long specified_event_processes;
 /* Allow for English spelling */
 static const char * neighbor_str = "neighbor";
 
+static void *current_bfd;
+
+
 static void
 bfd_handler(const vector_t *strvec)
 {
-	char *name;
-
 	global_data->have_bfd_config = true;
 
 	/* If we are not the bfd process, we don't need any more information */
 	if (!strvec)
 		return;
 
-	name = vector_slot(strvec, 1);
-
-	if (!alloc_bfd(name)) {
+	if (!(current_bfd = alloc_bfd(vector_slot(strvec, 1)))) {
 		skip_block(true);
 		return;
 	}
@@ -77,24 +76,23 @@ bfd_handler(const vector_t *strvec)
 static void
 bfd_nbrip_handler(const vector_t *strvec)
 {
-	bfd_t *bfd;
+	bfd_t *bfd = current_bfd;
 	sockaddr_t nbr_addr;
 
 	assert(strvec);
 	assert(bfd_data);
 
-	bfd = list_last_entry(&bfd_data->bfd, bfd_t, e_list);
-	assert(bfd);
-
 	if (!strcmp(vector_slot(strvec, 1), "neighbour_ip"))
 		neighbor_str = "neighbour";
 
-	if (inet_stosockaddr(strvec_slot(strvec, 1), BFD_CONTROL_PORT, &nbr_addr)) {
+	/* multihop may have already been specified */
+	if (inet_stosockaddr(strvec_slot(strvec, 1), bfd->multihop ? BFD_MULTIHOP_CONTROL_PORT : BFD_CONTROL_PORT, &nbr_addr)) {
 		report_config_error(CONFIG_GENERAL_ERROR,
 			    "Configuration error: BFD instance %s has"
 			    " malformed %s address %s, ignoring instance",
 			    bfd->iname, neighbor_str, strvec_slot(strvec, 1));
 		free_bfd(bfd);
+		current_bfd = NULL;
 		skip_block(false);
 		return;
 	}
@@ -106,15 +104,12 @@ bfd_nbrip_handler(const vector_t *strvec)
 static void
 bfd_srcip_handler(const vector_t *strvec)
 {
-	bfd_t *bfd;
+	bfd_t *bfd = current_bfd;
 	sockaddr_t src_addr;
 
 	assert(strvec);
 	assert(bfd_data);
 
-	bfd = list_last_entry(&bfd_data->bfd, bfd_t, e_list);
-	assert(bfd);
-
 	if (inet_stosockaddr(strvec_slot(strvec, 1), NULL, &src_addr)) {
 		report_config_error(CONFIG_GENERAL_ERROR,
 			    "Configuration error: BFD instance %s has"
@@ -129,15 +124,12 @@ bfd_srcip_handler(const vector_t *strvec)
 static void
 bfd_minrx_handler(const vector_t *strvec)
 {
-	bfd_t *bfd;
+	bfd_t *bfd = current_bfd;
 	unsigned value;
 
 	assert(strvec);
 	assert(bfd_data);
 
-	bfd = list_last_entry(&bfd_data->bfd, bfd_t, e_list);
-	assert(bfd);
-
 	if (!read_decimal_unsigned_strvec(strvec, 1, &value, BFD_MINRX_MIN * 1000, BFD_MINRX_MAX * 1000, 3, false)) {
 		report_config_error(CONFIG_GENERAL_ERROR, "Configuration error: BFD instance %s"
 			    " min_rx value %s is not valid (must be in range"
@@ -157,15 +149,12 @@ bfd_minrx_handler(const vector_t *strvec)
 static void
 bfd_mintx_handler(const vector_t *strvec)
 {
-	bfd_t *bfd;
+	bfd_t *bfd = current_bfd;
 	unsigned value;
 
 	assert(strvec);
 	assert(bfd_data);
 
-	bfd = list_last_entry(&bfd_data->bfd, bfd_t, e_list);
-	assert(bfd);
-
 	if (!read_decimal_unsigned_strvec(strvec, 1, &value, BFD_MINTX_MIN * 1000, BFD_MINTX_MAX * 1000, 3, false)) {
 		report_config_error(CONFIG_GENERAL_ERROR, "Configuration error: BFD instance %s"
 			    " min_tx value %s is not valid (must be in range"
@@ -185,15 +174,12 @@ bfd_mintx_handler(const vector_t *strvec)
 static void
 bfd_idletx_handler(const vector_t *strvec)
 {
-	bfd_t *bfd;
+	bfd_t *bfd = current_bfd;
 	unsigned value;
 
 	assert(strvec);
 	assert(bfd_data);
 
-	bfd = list_last_entry(&bfd_data->bfd, bfd_t, e_list);
-	assert(bfd);
-
 	if (!read_decimal_unsigned_strvec(strvec, 1, &value, BFD_IDLETX_MIN * 1000, BFD_IDLETX_MAX * 1000, 3, false)) {
 		report_config_error(CONFIG_GENERAL_ERROR, "Configuration error: BFD instance %s"
 			    " idle_tx value %s is not valid (must be in range"
@@ -213,15 +199,12 @@ bfd_idletx_handler(const vector_t *strvec)
 static void
 bfd_multiplier_handler(const vector_t *strvec)
 {
-	bfd_t *bfd;
+	bfd_t *bfd = current_bfd;
 	unsigned value;
 
 	assert(strvec);
 	assert(bfd_data);
 
-	bfd = list_last_entry(&bfd_data->bfd, bfd_t, e_list);
-	assert(bfd);
-
 	if (!read_unsigned_strvec(strvec, 1, &value, BFD_MULTIPLIER_MIN, BFD_MULTIPLIER_MAX, false))
 		report_config_error(CONFIG_GENERAL_ERROR, "Configuration error: BFD instance %s"
 			    " multiplier value %s not valid (must be in range"
@@ -234,28 +217,22 @@ bfd_multiplier_handler(const vector_t *strvec)
 static void
 bfd_passive_handler(__attribute__((unused)) const vector_t *strvec)
 {
-	bfd_t *bfd;
+	bfd_t *bfd = current_bfd;
 
 	assert(bfd_data);
 
-	bfd = list_last_entry(&bfd_data->bfd, bfd_t, e_list);
-	assert(bfd);
-
 	bfd->passive = true;
 }
 
 static void
 bfd_ttl_handler(const vector_t *strvec)
 {
-	bfd_t *bfd;
+	bfd_t *bfd = current_bfd;
 	unsigned value;
 
 	assert(strvec);
 	assert(bfd_data);
 
-	bfd = list_last_entry(&bfd_data->bfd, bfd_t, e_list);
-	assert(bfd);
-
 	if (!read_unsigned_strvec(strvec, 1, &value, 1, BFD_TTL_MAX, false))
 		report_config_error(CONFIG_GENERAL_ERROR, "Configuration error: BFD instance %s"
 			    " ttl/hoplimit value %s not valid (must be in range"
@@ -268,15 +245,12 @@ bfd_ttl_handler(const vector_t *strvec)
 static void
 bfd_maxhops_handler(const vector_t *strvec)
 {
-	bfd_t *bfd;
+	bfd_t *bfd = current_bfd;
 	int value;
 
 	assert(strvec);
 	assert(bfd_data);
 
-	bfd = list_last_entry(&bfd_data->bfd, bfd_t, e_list);
-	assert(bfd);
-
 	if (!read_int_strvec(strvec, 1, &value, -1, BFD_TTL_MAX, false))
 		report_config_error(CONFIG_GENERAL_ERROR, "Configuration error: BFD instance %s"
 			    " max_hops value %s not valid (must be in range"
@@ -286,15 +260,55 @@ bfd_maxhops_handler(const vector_t *strvec)
 		bfd->max_hops = value;
 }
 
+static void
+bfd_multihop_handler(const vector_t *strvec)
+{
+	bfd_t *bfd = current_bfd;
+	int value;
+
+	assert(strvec);
+	assert(bfd_data);
+
+	if (vector_size(strvec) == 1)
+		value = 1;
+	else {
+		value = check_true_false(vector_slot(strvec, 1));
+		if (value == -1) {
+			report_config_error(CONFIG_GENERAL_ERROR, "Configuration error: BFD instance %s"
+				    " multihop setting not valid - %s", bfd->iname, strvec_slot(strvec, 1));
+			return;
+		}
+	}
+
+	bfd->multihop = value;
+
+	/* Neighbour IP may have already been specified */
+#ifndef USE_SOCKADDR_STORAGE
+	if (bfd->nbr_addr.ss_family == AF_INET)
+		bfd->nbr_addr.in.sin_port = htons(atoi(bfd->multihop ? BFD_MULTIHOP_CONTROL_PORT : BFD_CONTROL_PORT));
+	else if (bfd->nbr_addr.ss_family == AF_INET6)
+		bfd->nbr_addr.in6.sin6_port = htons(atoi(bfd->multihop ? BFD_MULTIHOP_CONTROL_PORT : BFD_CONTROL_PORT));
+#else
+	if (bfd->nbr_addr.ss_family == AF_INET)
+		PTR_CAST(struct sockaddr_in, &bfd->nbr_addr)->sin_port = htons(atoi(bfd->multihop ? BFD_MULTIHOP_CONTROL_PORT : BFD_CONTROL_PORT));
+	else if (bfd->nbr_addr.ss_family == AF_INET6)
+		PTR_CAST(struct sockaddr_in6, &bfd->nbr_addr)->sin6_port = htons(atoi(bfd->multihop ? BFD_MULTIHOP_CONTROL_PORT : BFD_CONTROL_PORT));
+#endif
+}
+
 /* Checks for minimum configuration requirements */
 #ifdef _WITH_VRRP_
 static void
 bfd_vrrp_end_handler(void)
 {
-	vrrp_tracked_bfd_t *tbfd = list_last_entry(&vrrp_data->vrrp_track_bfds, vrrp_tracked_bfd_t, e_list);
+	vrrp_tracked_bfd_t *tbfd = current_bfd;
 
-	if (specified_event_processes && !__test_bit(DAEMON_VRRP, &specified_event_processes))
+	if (specified_event_processes && !__test_bit(DAEMON_VRRP, &specified_event_processes)) {
 		free_vrrp_tracked_bfd(tbfd);
+		return;
+	}
+
+	list_add_tail(&tbfd->e_list, &vrrp_data->vrrp_track_bfds);
 }
 #endif
 
@@ -302,19 +316,21 @@ bfd_vrrp_end_handler(void)
 static void
 bfd_checker_end_handler(void)
 {
-	checker_tracked_bfd_t *cbfd = list_last_entry(&check_data->track_bfds, checker_tracked_bfd_t, e_list);
+	checker_tracked_bfd_t *cbfd = current_bfd;
 
-	if (specified_event_processes && !__test_bit(DAEMON_CHECKERS, &specified_event_processes))
+	if (specified_event_processes && !__test_bit(DAEMON_CHECKERS, &specified_event_processes)) {
 		free_checker_bfd(cbfd);
+		return;
+	}
+
+	list_add_tail(&cbfd->e_list, &check_data->track_bfds);
 }
 #endif
 
 static void
 bfd_end_handler(void)
 {
-	bfd_t *bfd = list_last_entry(&bfd_data->bfd, bfd_t, e_list);
-
-	assert(bfd);
+	bfd_t *bfd = current_bfd;
 
 	if (!bfd->nbr_addr.ss_family) {
 		report_config_error(CONFIG_GENERAL_ERROR,
@@ -337,7 +353,7 @@ bfd_end_handler(void)
 		return;
 	}
 
-	if (find_bfd_by_addr(&bfd->nbr_addr, &bfd->src_addr)) {
+	if (find_bfd_by_addr(&bfd->nbr_addr, &bfd->src_addr, bfd->multihop)) {
 		if (bfd->src_addr.ss_family) {
 			char src_addr[INET6_ADDRSTRLEN];
 			strcpy(src_addr, inet_sockaddrtos(&bfd->src_addr));
@@ -377,6 +393,8 @@ bfd_end_handler(void)
 	bfd_checker_end_handler();
 #endif
 #endif
+
+	list_add_tail(&bfd->e_list, &bfd_data->bfd);
 }
 
 #ifdef _WITH_VRRP_
@@ -384,13 +402,10 @@ bfd_end_handler(void)
 static void
 bfd_vrrp_handler(const vector_t *strvec)
 {
-	const char *name;
-
 	if (!strvec)
 		return;
 
-	name = strvec_slot(strvec, 1);
-	alloc_vrrp_tracked_bfd(name, &vrrp_data->vrrp_track_bfds);
+	current_bfd = alloc_vrrp_tracked_bfd(strvec_slot(strvec, 1), &vrrp_data->vrrp_track_bfds);
 
 	specified_event_processes = 0;
 }
@@ -399,15 +414,12 @@ bfd_vrrp_handler(const vector_t *strvec)
 static void
 bfd_vrrp_weight_handler(const vector_t *strvec)
 {
-	vrrp_tracked_bfd_t *tbfd;
+	vrrp_tracked_bfd_t *tbfd = current_bfd;
 	int value;
 
 	assert(strvec);
 	assert(vrrp_data);
 
-	tbfd = list_last_entry(&vrrp_data->vrrp_track_bfds, vrrp_tracked_bfd_t, e_list);
-	assert(tbfd);
-
 	if (!read_int_strvec(strvec, 1, &value, -253, 253, true)) {
 		report_config_error(CONFIG_GENERAL_ERROR, "Configuration error: BFD instance %s"
 			    " weight value %s not valid (must be in range"
@@ -457,7 +469,8 @@ bfd_checker_handler(const vector_t *strvec)
 	INIT_LIST_HEAD(&cbfd->e_list);
 	INIT_LIST_HEAD(&cbfd->tracking_rs);
 	cbfd->bname = STRDUP(name);
-	list_add_tail(&cbfd->e_list, &check_data->track_bfds);
+
+	current_bfd = cbfd;
 
 	specified_event_processes = 0;
 }
@@ -494,21 +507,21 @@ init_bfd_keywords(bool active)
 	if (prog_type == PROG_TYPE_BFD || !active)
 #endif
 	{
-		install_keyword_root("bfd_instance", &bfd_handler, active);
-		install_sublevel_end_handler(bfd_end_handler);
+		install_keyword_root("bfd_instance", &bfd_handler, active, VPP &current_bfd);
+		install_level_end_handler(bfd_end_handler);
 		bfd_handlers = true;
 	}
 #ifndef _ONE_PROCESS_DEBUG_
 #ifdef _WITH_VRRP_
 	else if (prog_type == PROG_TYPE_VRRP) {
-		install_keyword_root("bfd_instance", &bfd_vrrp_handler, active);
-		install_sublevel_end_handler(bfd_vrrp_end_handler);
+		install_keyword_root("bfd_instance", &bfd_vrrp_handler, active, VPP &current_bfd);
+		install_level_end_handler(bfd_vrrp_end_handler);
 	}
 #endif
 #ifdef _WITH_LVS_
 	else if (prog_type == PROG_TYPE_CHECKER) {
-		install_keyword_root("bfd_instance", &bfd_checker_handler, active);
-		install_sublevel_end_handler(bfd_checker_end_handler);
+		install_keyword_root("bfd_instance", &bfd_checker_handler, active, VPP &current_bfd);
+		install_level_end_handler(bfd_checker_end_handler);
 	}
 #endif
 #endif
@@ -524,6 +537,7 @@ init_bfd_keywords(bool active)
 	install_keyword_conditional("ttl", &bfd_ttl_handler, bfd_handlers);
 	install_keyword_conditional("hoplimit", &bfd_ttl_handler, bfd_handlers);
 	install_keyword_conditional("max_hops", &bfd_maxhops_handler, bfd_handlers);
+	install_keyword_conditional("multihop", &bfd_multihop_handler, bfd_handlers);
 #ifdef _WITH_VRRP_
 	install_keyword_conditional("weight", &bfd_vrrp_weight_handler,
 #ifdef _ONE_PROCESS_DEBUG_
