@@ -76,7 +76,7 @@ dump_misc_check(FILE *fp, const checker_t *checker)
 	conf_write(fp, "   dynamic = %s", misck_checker->dynamic ? "YES" : "NO");
 	conf_write(fp, "   uid:gid = %u:%u", misck_checker->script.uid, misck_checker->script.gid);
 	ctime_r(&misck_checker->last_ran.tv_sec, time_str);
-	conf_write(fp, "   Last ran = %ld.%6.6ld (%.24s.%6.6ld)", misck_checker->last_ran.tv_sec, misck_checker->last_ran.tv_usec, time_str, misck_checker->last_ran.tv_usec);
+	conf_write(fp, "   Last ran = %" PRI_tv_sec ".%6.6" PRI_tv_usec " (%.24s.%6.6" PRI_tv_usec ")", misck_checker->last_ran.tv_sec, misck_checker->last_ran.tv_usec, time_str, misck_checker->last_ran.tv_usec);
 	conf_write(fp, "   Last status = %u", misck_checker->last_exit_code);
 }
 
@@ -133,7 +133,7 @@ misc_path_handler(__attribute__((unused)) const vector_t *strvec)
 	misc_checker_t *new_misck_checker = current_checker->data;
 
 	/* We need to allow quoted and escaped strings for the script and parameters */
-	strvec_qe = alloc_strvec_quoted_escaped(NULL);
+	strvec_qe = alloc_strvec_quoted(NULL);
 
 	set_script_params_array(strvec_qe, &new_misck_checker->script, 0);
 
@@ -209,7 +209,7 @@ install_misc_check_keyword(void)
 	install_keyword("MISC_CHECK", &misc_check_handler);
 	check_ptr = install_sublevel(VPP &current_checker);
 	install_checker_common_keywords(false);
-	install_keyword("misc_path", &misc_path_handler);
+	install_keyword_quoted("misc_path", &misc_path_handler);
 	install_keyword("misc_timeout", &misc_timeout_handler);
 	install_keyword("misc_dynamic", &misc_dynamic_handler);
 	install_keyword("user", &misc_user_handler);
@@ -403,7 +403,8 @@ misc_check_child_thread(thread_ref_t thread)
 				checker->cur_weight = 0;
 				misck_checker->last_exit_code = status;
 			}
-		}
+		} else	//  if (status != misck_checker->last_exit_code)
+			misck_checker->last_exit_code = status;
 	}
 	else if (WIFSIGNALED(wait_status)) {
 		if (misck_checker->state == SCRIPT_STATE_REQUESTING_TERMINATION && WTERMSIG(wait_status) == SIGTERM) {
