@@ -40,6 +40,7 @@
 #include "global_data.h"
 #include "scheduler.h"
 #include "utils.h"
+#include "process.h"
 
 static int dir_wd, file_wd;
 
@@ -73,7 +74,7 @@ reload_timer_thread(__attribute__((unused)) thread_ref_t thread)
 	    global_data->reload_date_specified)
 		unlink(global_data->reload_time_file);
 
-	kill(getpid(), SIGHUP);
+	kill(our_pid, SIGHUP);
 	/*
 	or
 	reload_config();
@@ -217,7 +218,7 @@ parse_datetime(const char *timestr, bool *date_specified)
 static void
 read_file(void)
 {
-	FILE *fp = fopen(global_data->reload_time_file, "r");
+	FILE *fp = fopen(global_data->reload_time_file, "re");
 	size_t len;
 	time_t reload_time;
 	char	time_buf[21];
