@@ -23,6 +23,16 @@
 #include "config.h"
 
 #include <stdio.h>
+#if defined HAVE_CLOSE_RANGE && HAVE_DECL_CLOSE_RANGE_CLOEXEC
+#if !defined USE_CLOSE_RANGE_SYSCALL && !defined _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
+#include <linux/close_range.h>
+#else
+#include <fcntl.h>
+#endif
+#include <unistd.h>
+
 #include "scheduler.h"
 #include "snmp.h"
 #include "logger.h"
@@ -456,9 +466,19 @@ snmp_unregister_mib(oid *myoid, size_t len)
 void
 snmp_agent_init(const char *snmp_socket_name, bool base_mib)
 {
+#if !defined HAVE_CLOSE_RANGE || !HAVE_DECL_CLOSE_RANGE_CLOEXEC
+	uint64_t fds[2][16];
+	unsigned max_fd;
+	size_t i;
+#endif
+
 	if (snmp_running)
 		return;
 
+#if !defined HAVE_CLOSE_RANGE || !HAVE_DECL_CLOSE_RANGE_CLOEXEC
+	get_open_fds(fds[0], sizeof(fds[0]) / sizeof(fds[0][0]));
+#endif
+
 	log_message(LOG_INFO, "Starting SNMP subagent");
 	netsnmp_enable_subagent();
 	snmp_disable_log();
@@ -509,6 +529,27 @@ snmp_agent_init(const char *snmp_socket_name, bool base_mib)
 	/* Set up the fd threads */
 	snmp_epoll_info(master);
 
+#if defined HAVE_CLOSE_RANGE && HAVE_DECL_CLOSE_RANGE_CLOEXEC
+	/* This assumes that child processes should only have stdin, stdout and stderr open */
+	close_range(STDERR_FILENO + 1, ~0U, CLOSE_RANGE_CLOEXEC);
+#else
+	max_fd = get_open_fds(fds[1], sizeof(fds[1]) / sizeof(fds[1][0]));
+
+	for (i = 0; i < sizeof(fds[0]) / sizeof(fds[0][0]) && i * 64 <= max_fd; i++) {
+		if (fds[0][i] != fds[1][i]) {
+			uint64_t fds_diff = fds[0][i] ^ fds[1][i], bit_mask;
+			unsigned j;
+			for (bit_mask = 1, j = i * 64; j < (i + 1) * 64 && j <= max_fd; j++, bit_mask <<= 1) {
+				if (j <= STDERR_FILENO)
+					continue;
+
+				if (fds_diff & bit_mask)
+					fcntl(j, F_SETFD, fcntl(j, F_GETFD) | FD_CLOEXEC);
+			}
+		}
+	}
+#endif
+
 	snmp_running = true;
 }
 
