@@ -29,7 +29,7 @@
 
 #include "config.h"
 
-#include <sys/socket.h>
+#include <sys/uio.h>
 #include <linux/netlink.h>
 #include <linux/connector.h>
 #include <linux/cn_proc.h>
@@ -293,7 +293,7 @@ read_procs(list_head_t *processes)
 		if (vrrp_data->vrrp_use_process_cmdline) {
 			snprintf(cmdline, sizeof(cmdline), "/proc/%.*s/cmdline", PID_MAX_DIGITS, ent->d_name);
 
-			if ((fd = open(cmdline, O_RDONLY)) == -1)
+			if ((fd = open(cmdline, O_RDONLY | O_CLOEXEC)) == -1)
 				continue;
 
 			/* Read max name len + null byte + 1 extra char */
@@ -307,7 +307,7 @@ read_procs(list_head_t *processes)
 		if (vrrp_data->vrrp_use_process_comm) {
 			snprintf(cmdline, sizeof(cmdline), "/proc/%.*s/stat", PID_MAX_DIGITS, ent->d_name);
 
-			if ((fd = open(cmdline, O_RDONLY)) == -1)
+			if ((fd = open(cmdline, O_RDONLY | O_CLOEXEC)) == -1)
 				continue;
 
 			len = read(fd, stat_buf, sizeof(stat_buf) - 1);
@@ -432,7 +432,7 @@ check_process(pid_t pid, char *comm, tracked_process_instance_t *tpi)
 		if (vrrp_data->vrrp_use_process_cmdline) {
 			snprintf(cmdline, sizeof(cmdline), "/proc/%d/cmdline", pid);
 
-			if ((fd = open(cmdline, O_RDONLY)) == -1) {
+			if ((fd = open(cmdline, O_RDONLY | O_CLOEXEC)) == -1) {
 #ifdef _TRACK_PROCESS_DEBUG_
 				if (do_track_process_debug_detail)
 					log_message(LOG_INFO, "check_process failed to open %s, errno %d", cmdline, errno);
@@ -461,7 +461,7 @@ check_process(pid_t pid, char *comm, tracked_process_instance_t *tpi)
 		if (vrrp_data->vrrp_use_process_comm) {
 			snprintf(cmdline, sizeof(cmdline), "/proc/%d/comm", pid);
 
-			if ((fd = open(cmdline, O_RDONLY)) == -1) {
+			if ((fd = open(cmdline, O_RDONLY | O_CLOEXEC)) == -1) {
 #ifdef _TRACK_PROCESS_DEBUG_
 				if (do_track_process_debug_detail)
 					log_message(LOG_INFO, "check_process failed to open %s, errno %d", cmdline, errno);
@@ -778,7 +778,7 @@ nl_connect(void)
 
 	sa_nl.nl_family = AF_NETLINK;
 	sa_nl.nl_groups = CN_IDX_PROC;
-	sa_nl.nl_pid = getpid();
+	sa_nl.nl_pid = our_pid;
 
 	rc = bind(nl_sd, PTR_CAST(struct sockaddr, &sa_nl), sizeof(sa_nl));
 	if (rc == -1) {
@@ -796,26 +796,16 @@ nl_connect(void)
 static int set_proc_ev_listen(int nl_sd, bool enable)
 {
 	int rc;
-	struct __attribute__ ((aligned(NLMSG_ALIGNTO))) {
-		struct nlmsghdr nl_hdr;
-		struct __attribute__ ((__packed__)) {
-			struct cn_msg cn_msg;
-			enum proc_cn_mcast_op cn_mcast;
-		};
-	} nlcn_msg;
+	enum proc_cn_mcast_op cn_mcast = enable ? PROC_CN_MCAST_LISTEN : PROC_CN_MCAST_IGNORE;
+	struct cn_msg cn_msg = { .id.idx = CN_IDX_PROC, .id.val = CN_VAL_PROC, .len = sizeof(cn_mcast) };
+	struct nlmsghdr nl_hdr = { .nlmsg_type = NLMSG_DONE, .nlmsg_pid = our_pid, .nlmsg_len = NLMSG_LENGTH(sizeof(cn_msg) + sizeof(cn_mcast)) };
+	struct iovec iov[3] = {
+		{ .iov_base = &nl_hdr, .iov_len = sizeof(nl_hdr) },
+		{ .iov_base = &cn_msg, .iov_len = sizeof(cn_msg) },
+		{ .iov_base = &cn_mcast, .iov_len = sizeof(cn_mcast) },
+	};
 
-	memset(&nlcn_msg, 0, sizeof(nlcn_msg));
-	nlcn_msg.nl_hdr.nlmsg_len = sizeof(nlcn_msg);
-	nlcn_msg.nl_hdr.nlmsg_pid = getpid();
-	nlcn_msg.nl_hdr.nlmsg_type = NLMSG_DONE;
-
-	nlcn_msg.cn_msg.id.idx = CN_IDX_PROC;
-	nlcn_msg.cn_msg.id.val = CN_VAL_PROC;
-	nlcn_msg.cn_msg.len = sizeof(enum proc_cn_mcast_op);
-
-	nlcn_msg.cn_mcast = enable ? PROC_CN_MCAST_LISTEN : PROC_CN_MCAST_IGNORE;
-
-	rc = send(nl_sd, &nlcn_msg, sizeof(nlcn_msg), 0);
+	rc = writev(nl_sd, iov, 3);
 	if (rc == -1) {
 		log_message(LOG_INFO, "Failed to %s process event listen - errno %d - %m", enable ? "set" : "clear", errno);
 		return -1;
