@@ -286,6 +286,7 @@ typedef struct _data {
 	int				vrrp_rx_bufs_multiples;
 	unsigned			vrrp_startup_delay;
 	bool				log_unknown_vrids;
+	bool				vrrp_owner_ignore_adverts;
 #ifdef _HAVE_VRRP_VMAC_
 	const char			*vmac_prefix;
 	const char			*vmac_addr_prefix;
@@ -298,6 +299,9 @@ typedef struct _data {
 	const char			*iproute_usr_dir;
 	const char			*iproute_etc_dir;
 #endif
+	const char			*state_dump_file;
+	const char			*stats_dump_file;
+	const char			*json_dump_file;
 } data_t;
 
 /* Global vars exported */
