@@ -45,7 +45,7 @@ int header_generic(struct variable *, oid *, size_t *, int,
 #endif
 #undef FREE
 
-#include "list.h"
+#include "list_head.h"
 
 #define SNMP_DEFAULT_NETWORK_SOCKET	"udp:localhost:705"
 
@@ -53,6 +53,12 @@ int header_generic(struct variable *, oid *, size_t *, int,
 #define SNMPTRAP_OID 1, 3, 6, 1, 6, 3, 1, 1, 4, 1, 0
 #define GLOBAL_OID {KEEPALIVED_OID, 1}
 
+/* A TruthValue has value 1 for true and 2 for false */
+#define SNMP_TruthValue(x)	(1 + !(x))
+
+/* A TruthValue has value 1 for AF_INET and 2 for AF_INET6 */
+#define SNMP_InetAddressType(x)	((x) == AF_INET ? 1 : 2)
+
 typedef union {
 	unsigned char uc;
 	unsigned long u;
@@ -67,12 +73,12 @@ typedef union {
 } snmp_ret_t;
 
 extern unsigned long snmp_scope(int ) __attribute__ ((const));
-extern void* snmp_header_list_table(struct variable *, oid *, size_t *,
-				    int, size_t *, WriteMethod **,
-				    list);
-extern element snmp_find_element(struct variable *, oid *, size_t *,
-				 int, size_t *, WriteMethod **,
-				 list, size_t);
+extern list_head_t *snmp_header_list_head_table(struct variable *, oid *, size_t *,
+						int, size_t *, WriteMethod **,
+						list_head_t *);
+extern list_head_t *snmp_find_element(struct variable *, oid *, size_t *,
+				      int, size_t *, WriteMethod **,
+				      list_head_t *, size_t, size_t);
 extern void snmp_agent_init(const char *, bool);
 extern void snmp_register_mib(oid *, size_t, const char *,
 			      struct variable *, size_t, size_t);
