@@ -84,6 +84,28 @@ enum vrrp_flags_bits {
 #endif
 };
 
+typedef enum vrrp_rlflags {
+	VRRP_RLFLAG_INVALID_TTL		=     0x1,
+	VRRP_RLFLAG_WRONG_VERSION	=     0x2,
+	VRRP_RLFLAG_NOT_ADVERTISEMENT	=     0x4,
+	VRRP_RLFLAG_INCOMPLETE_PACKET	=     0x8,
+	VRRP_RLFLAG_NO_VIPS		=    0x10,
+	VRRP_RLFLAG_WRONG_ADDR_COUNT	=    0x20,
+	VRRP_RLFLAG_VIPS_MISMATCH	=    0x40,
+	VRRP_RLFLAG_WRONG_AUTH		=    0x80,
+	VRRP_RLFLAG_BAD_AUTH		=   0x100,
+	VRRP_RLFLAG_BAD_AH_HEADER	=   0x200,
+	VRRP_RLFLAG_BAD_IP_VERSION	=   0x400,
+	VRRP_RLFLAG_BAD_LENGTH		=   0x800,
+	VRRP_RLFLAG_WRONG_AUTH_PASSWD	=  0x1000,
+	VRRP_RLFLAG_ADV_INTVL_MISMATCH	=  0x2000,
+	VRRP_RLFLAG_BAD_CHECKSUM	=  0x4000,
+	VRRP_RLFLAG_UNI_MULTICAST_ERR	=  0x8000,
+	VRRP_RLFLAG_UNKNOWN_UNICAST_SRC	= 0x10000,
+	VRRP_RLFLAG_TTL_NOT_IN_RANGE	= 0x20000,
+	VRRP_RLFLAG_OWNER_IGNORE_ADVER	= 0x40000,
+} vrrp_rlflags_t;
+
 typedef struct _vrrphdr {			/* rfc2338.5.1 */
 	uint8_t			vers_type;	/* 0-3=type, 4-7=version */
 	uint8_t			vrid;		/* virtual router id */
@@ -95,7 +117,7 @@ typedef struct _vrrphdr {			/* rfc2338.5.1 */
 			uint8_t adver_int;	/* advertisement interval (in sec) */
 		} v2;
 		struct {
-			uint16_t adver_int;	/* advertisement interval (in centi-sec (100ms)) */
+			uint16_t adver_int;	/* advertisement interval (in centi-sec (10ms)) */
 		} v3;
 	};
 	uint16_t		chksum;		/* checksum (ip-like one) */
@@ -135,6 +157,9 @@ typedef struct {
 #define VRRP_GARP_REFRESH	0		/* Default interval for refresh gratuitous arp (0 = none) */
 #define VRRP_GARP_REFRESH_REP	1		/* Default repeat value for refresh gratuitous arp */
 
+#define V3_PKT_ADVER_INT_NTOH(ai)	(ntohs(ai) & 0xFFF)
+#define V3_PKT_ADVER_INT_HTON(ai)	(htons(ai & 0xFFF))
+
 /*
  * parameters per vrrp sync group. A vrrp_sync_group is a set
  * of VRRP instances that need to be state sync together.
@@ -246,6 +271,17 @@ typedef struct _unicast_peer_t {
 	list_head_t		e_list;
 } unicast_peer_t;
 
+typedef enum vrrp_fault_fl {
+	VRRP_FAULT_FL_TRACKER = 0,
+	VRRP_FAULT_FL_INTERFACE_DOWN,
+#ifdef _HAVE_VRRP_VMAC_
+	VRRP_FAULT_FL_BASE_INTERFACE_DOWN,
+#endif /* _HAVE_VRRP_VMAC_ */
+	VRRP_FAULT_FL_DUPLICATE_VRID,
+	VRRP_FAULT_FL_NO_SOURCE_IP,
+	VRRP_FAULT_FL_CONFIG_ERROR,
+} vrrp_fault_fl_t;
+
 /* parameters per virtual router -- rfc2338.6.1.2 */
 typedef struct _vrrp_t {
 	sa_family_t		family;			/* AF_INET|AF_INET6 */
@@ -258,6 +294,7 @@ typedef struct _vrrp_t {
 #endif
 	unsigned		strict_mode;		/* Enforces strict VRRP compliance */
 	unsigned long		flags;
+	vrrp_rlflags_t		rlflags;		/* Flags for rate-limiting log messages */
 #ifdef _HAVE_VRRP_VMAC_
 	char			vmac_ifname[IFNAMSIZ];	/* Name of VRRP VMAC interface */
 	u_char			ll_addr[ETH_ALEN];	/* Override MAC address */
@@ -278,14 +315,15 @@ typedef struct _vrrp_t {
 	list_head_t		track_bfd;		/* tracked_bfd_t - BFD instance state we monitor */
 #endif
 	unsigned		num_config_faults;	/* Number of configuration errors */
-	unsigned		num_script_if_fault;	/* Number of scripts and interfaces in fault state */
+	unsigned		num_track_fault;	/* Number of trackers (script, bfd...) in fault state */
+	unsigned long		flags_if_fault;		/* Flags of interface fault */
 	unsigned		num_script_init;	/* Number of scripts in init state */
 	bool			notifies_sent;		/* Set when initial notifies have been sent */
 	bool			multicast_pkt;		/* Last IPv6 packet received was multicast */
 	sockaddr_t		saddr;			/* Src IP address to use in VRRP IP header */
 	sockaddr_t		pkt_saddr;		/* Src IP address received in VRRP IP header */
 	sockaddr_t		mcast_daddr;		/* Multicast destination address */
-	int			rx_ttl_hop_limit;	/* Received TTL/hop limit returned */
+	int			rx_ttl_hl;		/* Received TTL/hop limit returned */
 	list_head_t		unicast_peer;		/* unicast_peer_t - peers to send unicast advert to */
 	int			ttl;			/* TTL to send packet with if unicasting */
 #ifdef _WITH_UNICAST_CHKSUM_COMPAT_
@@ -306,6 +344,7 @@ typedef struct _vrrp_t {
 	unsigned		down_timer_adverts;	/* Number of adverts missed before backup takes over as master */
 	unsigned		lower_prio_no_advert;	/* Don't send advert after lower prio advert received */
 	unsigned		higher_prio_send_advert; /* Send advert after higher prio advert received */
+	unsigned		owner_ignore_adverts;	/* Set if an address owner should ignore received adverts */
 #ifdef _HAVE_VRRP_VMAC_
 	timeval_t		vmac_garp_intvl;	/* Interval between GARPs on each VMAC */
 #endif
@@ -333,6 +372,10 @@ typedef struct _vrrp_t {
 	timeval_t		last_advert_sent;	/* Time of sending last advert */
 	size_t			kernel_rx_buf_size;	/* Socket receive buffer size */
 
+	unsigned		rogue_counter;		/* Used if we are address owner and another */
+	thread_ref_t		rogue_timer_thread;	/* system advertises it is the address owner */
+	unsigned		rogue_adver_int;
+
 #ifdef _WITH_FIREWALL_
 	unsigned		accept;			/* Allow the non-master owner to process
 							 * the packets destined to VIP. */
@@ -421,11 +464,13 @@ typedef struct _vrrp_t {
 #define VRRP_EVENT_BACKUP_PRIORITY_CHANGE 1002	/* Dummy state for sending event notify */
 
 /* VRRP packet handling */
-#define VRRP_PACKET_OK       0
-#define VRRP_PACKET_KO       1
-#define VRRP_PACKET_DROP     2
-#define VRRP_PACKET_NULL     3
-#define VRRP_PACKET_OTHER    4	/* Multiple VRRP on LAN, Identify "other" VRRP */
+enum vrrp_packet_status {
+	VRRP_PACKET_OK,
+	VRRP_PACKET_KO,
+	VRRP_PACKET_DROP,
+	VRRP_PACKET_NULL,
+	VRRP_PACKET_OTHER	/* Multiple VRRP on LAN, Identify "other" VRRP */
+};
 
 /* VRRP Packet fixed length */
 #define VRRP_AUTH_LEN		8
@@ -434,7 +479,7 @@ typedef struct _vrrp_t {
 
 /* We have to do some reduction of the calculation for VRRPv3 in order not to overflow a uint32; 625 / 16 == TIMER_CENTI_HZ / 256 */
 #define VRRP_TIMER_SKEW_CALC(svr, pri_val) ((svr)->version == VRRP_VERSION_3 ? (((pri_val) * ((svr)->master_adver_int / TIMER_CENTI_HZ) * 625U) / 16U) : ((pri_val) * TIMER_HZ/256U))
-#define VRRP_TIMER_SKEW(svr)		VRRP_TIMER_SKEW_CALC(svr, 256U - (svr)->effective_priority)
+#define VRRP_TIMER_SKEW(svr)		VRRP_TIMER_SKEW_CALC(svr, 256U - ((svr)->base_priority == VRRP_PRIO_OWNER ? VRRP_PRIO_OWNER : (svr)->effective_priority))
 #define VRRP_TIMER_SKEW_MIN(svr)	VRRP_TIMER_SKEW_CALC(svr, 1)
 #define VRRP_MS_DOWN_TIMER(XX)		((XX)->down_timer_adverts * (XX)->master_adver_int + VRRP_TIMER_SKEW(XX))
 
@@ -450,7 +495,7 @@ typedef struct _vrrp_t {
 #endif
 #define VRRP_PKT_SADDR(V) (((V)->saddr.ss_family) ? ((struct sockaddr_in *) &(V)->saddr)->sin_addr.s_addr : IF_ADDR(VRRP_CONFIGURED_IFP(V)))
 
-#define VRRP_ISUP(V)		(!(V)->num_script_if_fault)
+#define VRRP_ISUP(V)		(!(V)->flags_if_fault)
 
 
 /* Configuration summary flags */
