@@ -219,7 +219,7 @@ set_disable_local_igmp(void)
 	int fd;
 	ssize_t len;
 
-	if ((fd = open(igmp_link_local_mcast_reports, O_RDWR)) == -1) {
+	if ((fd = open(igmp_link_local_mcast_reports, O_RDWR | O_CLOEXEC)) == -1) {
 		log_message(LOG_INFO, "Unable to open %s - errno %d", igmp_link_local_mcast_reports, errno);
 		global_data->disable_local_igmp = false;
 		return;
@@ -252,7 +252,7 @@ reset_disable_local_igmp(void)
 	int fd;
 
 	if (sav_igmp_link_local_mcast_reports == '1') {
-		fd = open(igmp_link_local_mcast_reports, O_RDWR);
+		fd = open(igmp_link_local_mcast_reports, O_RDWR | O_CLOEXEC);
 		if (fd == -1 || write(fd, &sav_igmp_link_local_mcast_reports, 1) == -1)
 			log_message(LOG_INFO, "Unable to write %s - errno %d", igmp_link_local_mcast_reports, errno);
 		if (fd != -1)
@@ -336,6 +336,10 @@ vrrp_terminate_phase2(int exit_status)
 	 */
 	log_stopping();
 
+#ifdef DO_STACKSIZE
+	get_stacksize(true);
+#endif
+
 #ifdef ENABLE_LOG_TO_FILE
 	if (log_file_name)
 		close_log_file();
@@ -706,8 +710,12 @@ start_vrrp(data_t *prev_global_data)
 		dump_data_vrrp(NULL);
 
 	/* Set the process priority and non swappable if configured */
-	set_process_priorities(global_data->vrrp_realtime_priority, global_data->max_auto_priority, global_data->min_auto_priority_delay,
-			       global_data->vrrp_rlimit_rt, global_data->vrrp_process_priority, global_data->vrrp_no_swap ? 4096 : 0);
+	if (reload)
+		restore_priority(global_data->vrrp_realtime_priority, global_data->max_auto_priority, global_data->min_auto_priority_delay,
+				       global_data->vrrp_rlimit_rt, global_data->vrrp_process_priority, global_data->vrrp_no_swap ? VRRP_STACK_SIZE : 0);
+	else
+		set_process_priorities(global_data->vrrp_realtime_priority, global_data->max_auto_priority, global_data->min_auto_priority_delay,
+				       global_data->vrrp_rlimit_rt, global_data->vrrp_process_priority, global_data->vrrp_no_swap ? VRRP_STACK_SIZE : 0);
 
 	/* Set the process cpu affinity if configured */
 	set_process_cpu_affinity(&global_data->vrrp_cpu_mask, "vrrp");
@@ -762,7 +770,7 @@ static void
 sigusr1_vrrp(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 {
 	log_message(LOG_INFO, "Printing VRRP data for process(%d) on signal",
-		    getpid());
+		    our_pid);
 	thread_add_event(master, print_vrrp_data, NULL, 0);
 }
 
@@ -770,7 +778,7 @@ static void
 sigusr2_vrrp(__attribute__((unused)) void *v, int sig)
 {
 	log_message(LOG_INFO, "Printing %sVRRP stats for process(%d) on signal",
-		    sig == SIGSTATS_CLEAR ? "and clearing " : "", getpid());
+		    sig == SIGSTATS_CLEAR ? "and clearing " : "", our_pid);
 	thread_add_event(master, print_vrrp_stats, NULL, sig);
 }
 
@@ -824,7 +832,7 @@ reload_vrrp_thread(__attribute__((unused)) thread_ref_t thread)
 	log_message(LOG_INFO, "Reloading");
 
 	/* Use standard scheduling while reloading */
-	reset_process_priorities();
+	reset_priority();
 
 #ifndef _ONE_PROCESS_DEBUG_
 	save_config(false, "vrrp", dump_data_vrrp);
@@ -1053,6 +1061,8 @@ start_vrrp_child(void)
 		return 0;
 	}
 
+	our_pid = getpid();
+
 #ifdef _WITH_PROFILING_
 	/* See https://lists.gnu.org/archive/html/bug-gnu-utils/2001-09/msg00047.html for details */
 	monstartup ((u_long) &_start, (u_long) &etext);
@@ -1062,7 +1072,7 @@ start_vrrp_child(void)
 
 	/* Check our parent hasn't already changed since the fork */
 	if (main_pid != getppid())
-		kill(getpid(), SIGTERM);
+		kill(our_pid, SIGTERM);
 
 #ifdef _WITH_PERF_
 	if (perf_run == PERF_ALL)
@@ -1108,6 +1118,10 @@ start_vrrp_child(void)
 				global_data->instance_name);
 #endif
 
+#ifdef DO_STACKSIZE
+	get_stacksize(false);
+#endif
+
 #ifdef _MEM_CHECK_
 	mem_log_init(PROG_VRRP, "VRRP Child process");
 #endif
@@ -1117,7 +1131,7 @@ start_vrrp_child(void)
 	/* Clear any child finder functions set in parent */
 	set_child_finder_name(NULL);
 
-	/* Create an independant file descriptor for the shared config file */
+	/* Create an independent file descriptor for the shared config file */
 	separate_config_file();
 
 	/* Child process part, write pidfile */
