@@ -26,8 +26,8 @@
 #include <sys/types.h>
 
 /* Prototypes */
-extern size_t extract_content_length(char *buffer, size_t size);
-extern int extract_status_code(char *buffer, size_t size);
-extern char *extract_html(char *buffer, size_t size_buffer);
+extern size_t extract_content_length(const char *buffer, size_t size);
+extern int extract_status_code(const char *buffer, size_t size);
+extern const char *extract_html(const char *buffer, size_t size_buffer) __attribute__ ((pure));
 
 #endif
