@@ -761,8 +761,8 @@ mem_log_init_common(struct mem_domain *mem, const char* prog_name, const char *b
 		return;
 	}
 
-	snprintf(log_name, log_name_len, "%s/%s_mem.%d.log", tmp_dir, prog_name, getpid());
-	mem->log_op = fopen_safe(log_name, "w");
+	snprintf(log_name, log_name_len, "%s/%s_mem.%d.log", tmp_dir, prog_name, our_pid);
+	mem->log_op = fopen_safe(log_name, "we");
 	if (mem->log_op == NULL) {
 		log_message(LOG_INFO, "Unable to open %s for appending", log_name);
 		mem->log_op = stderr;
