@@ -22,6 +22,12 @@
 
 #include "config.h"
 
+#ifdef HAVE_CLOSE_RANGE
+#if !defined USE_CLOSE_RANGE_SYSCALL && !defined _GNU_SOURCE
+#define _GNU_SOURCE         /* for close_range() in unistd.h */
+#endif
+#include <linux/close_range.h>
+#endif
 #include <unistd.h>
 #include <stdlib.h>
 #include <fcntl.h>
@@ -199,6 +205,20 @@ system_call_script(thread_master_t *m, thread_func_t func, void * arg, unsigned
 	skip_mem_dump();
 #endif
 
+#ifdef HAVE_CLOSE_RANGE
+	 /* We don't want anything past stderr here. This is belt
+	  * and braces really, since all file desriptors should
+	  * have FD_CLOEXEC set. CLOSE_RANGE_CLOEXEC uses fewer
+	  * kernel resources that CLOSE_RANGE_UNSHARE. */
+	close_range(STDERR_FILENO + 1, ~0U,
+#if HAVE_DECL_CLOSE_RANGE_CLOEXEC
+			    CLOSE_RANGE_CLOEXEC
+#else
+			    CLOSE_RANGE_UNSHARE
+#endif
+					       );
+#endif
+
 	if (set_script_env(script->uid, script->gid))
 		exit(0);
 
@@ -239,7 +259,7 @@ system_call_script(thread_master_t *m, thread_func_t func, void * arg, unsigned
 		}
 
 		if (WIFSIGNALED(retval))
-			kill(getpid(), WTERMSIG(retval));
+			kill(our_pid, WTERMSIG(retval));
 	}
 
 	exit(0); /* Script errors aren't server errors */
@@ -838,7 +858,7 @@ check_script_secure(notify_script_t *script,
 
 		/* Check the parent didn't die in the window when PDEATHSIG was not set */
 		if (!__test_bit(CONFIG_TEST_BIT, &debug) && main_pid != getppid())
-			kill(getpid(), SIGTERM);
+			kill(our_pid, SIGTERM);
 	}
 
 	if (!real_path) {
@@ -939,6 +959,7 @@ static void
 set_pwnam_buf_len(void)
 {
 	long buf_len;
+	struct stat statbuf;
 
 	/* Get buffer length needed for getpwnam_r/getgrnam_r */
 	if ((buf_len = sysconf(_SC_GETPW_R_SIZE_MAX)) == -1)
@@ -948,6 +969,12 @@ set_pwnam_buf_len(void)
 	if ((buf_len = sysconf(_SC_GETGR_R_SIZE_MAX)) != -1 &&
 	    (size_t)buf_len > getpwnam_buf_len)
 		getpwnam_buf_len = (size_t)buf_len;
+
+	/* In case there is an excessive number of members of the group, which
+	 * could cause the default buffer size to overflow, a reasonable upper
+	 * bound for the buffer size is the size of the group file itself. */
+	if (!stat("/etc/group", &statbuf) && (unsigned long)statbuf.st_size > getpwnam_buf_len)
+		getpwnam_buf_len = statbuf.st_size;
 }
 
 static bool
