@@ -1,38 +1,32 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
   Red Black Trees
   (C) 1999  Andrea Arcangeli <andrea@suse.de>
   (C) 2002  David Woodhouse <dwmw2@infradead.org>
   (C) 2012  Michel Lespinasse <walken@google.com>
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
   linux/lib/rbtree.c
 */
 
-/* This is updated to Linux v4.18.10 lib/rbtree.c
+/* This is updated to Linux v5.15-rc4 lib/rbtree.c
  *   with the addition of the defined helper "functions" in rbtree.h
  */
 
+/* Allow including rbtree.h directly */
+#define _RBTREE_KA_H
+
 #include "config.h"
 
 #include <stdlib.h>
 #include "rbtree_augmented.h"
 #include"align.h"
 
+#define EXPORT_SYMBOL(xxx)
+#define UNUSED	__attribute__((unused))
+
 /*
- * red-black trees properties:  http://en.wikipedia.org/wiki/Rbtree
+ * red-black trees properties:  https://en.wikipedia.org/wiki/Rbtree
  *
  *  1) A node is either red or black
  *  2) The root is black
@@ -82,7 +76,7 @@ static inline void rb_set_black(struct rb_node *rb)
 
 static inline struct rb_node *rb_red_parent(struct rb_node *red)
 {
-	return PTR_CAST(struct rb_node, (void *)red->__rb_parent_color);
+	return PTR_CAST(struct rb_node, red->__rb_parent_color);
 }
 
 /*
@@ -102,14 +96,10 @@ __rb_rotate_set_parents(struct rb_node *old, struct rb_node *new,
 
 static __always_inline void
 __rb_insert(struct rb_node *node, struct rb_root *root,
-	    bool newleft, struct rb_node **leftmost,
 	    void (*augment_rotate)(struct rb_node *old, struct rb_node *new))
 {
 	struct rb_node *parent = rb_red_parent(node), *gparent, *tmp;
 
-	if (newleft)
-		*leftmost = node;
-
 	while (true) {
 		/*
 		 * Loop invariant: node is red.
@@ -435,6 +425,7 @@ void __rb_erase_color(struct rb_node *parent, struct rb_root *root,
 {
 	____rb_erase_color(parent, root, augment_rotate);
 }
+EXPORT_SYMBOL(__rb_erase_color);
 
 /*
  * Non-augmented rbtree manipulation functions.
@@ -443,9 +434,9 @@ void __rb_erase_color(struct rb_node *parent, struct rb_root *root,
  * out of the rb_insert_color() and rb_erase() function definitions.
  */
 
-static inline void dummy_propagate(__attribute__((unused)) struct rb_node *node, __attribute__((unused)) struct rb_node *stop) {}
-static inline void dummy_copy(__attribute__((unused)) struct rb_node *old, __attribute__((unused)) struct rb_node *new) {}
-static inline void dummy_rotate(__attribute__((unused)) struct rb_node *old, __attribute__((unused)) struct rb_node *new) {}
+static inline void dummy_propagate(UNUSED struct rb_node *node, UNUSED struct rb_node *stop) {}
+static inline void dummy_copy(UNUSED struct rb_node *old, UNUSED struct rb_node *new) {}
+static inline void dummy_rotate(UNUSED struct rb_node *old, UNUSED struct rb_node *new) {}
 
 static const struct rb_augment_callbacks dummy_callbacks = {
 	.propagate = dummy_propagate,
@@ -455,33 +446,18 @@ static const struct rb_augment_callbacks dummy_callbacks = {
 
 void rb_insert_color(struct rb_node *node, struct rb_root *root)
 {
-	__rb_insert(node, root, false, NULL, dummy_rotate);
+	__rb_insert(node, root, dummy_rotate);
 }
+EXPORT_SYMBOL(rb_insert_color);
 
 void rb_erase(struct rb_node *node, struct rb_root *root)
 {
 	struct rb_node *rebalance;
-	rebalance = __rb_erase_augmented(node, root,
-					 NULL, &dummy_callbacks);
+	rebalance = __rb_erase_augmented(node, root, &dummy_callbacks);
 	if (rebalance)
 		____rb_erase_color(rebalance, root, dummy_rotate);
 }
-
-void rb_insert_color_cached(struct rb_node *node,
-			    struct rb_root_cached *root, bool leftmost)
-{
-	__rb_insert(node, &root->rb_root, leftmost,
-		    &root->rb_leftmost, dummy_rotate);
-}
-
-void rb_erase_cached(struct rb_node *node, struct rb_root_cached *root)
-{
-	struct rb_node *rebalance;
-	rebalance = __rb_erase_augmented(node, &root->rb_root,
-					 &root->rb_leftmost, &dummy_callbacks);
-	if (rebalance)
-		____rb_erase_color(rebalance, &root->rb_root, dummy_rotate);
-}
+EXPORT_SYMBOL(rb_erase);
 
 /*
  * Augmented rbtree manipulation functions.
@@ -491,16 +467,16 @@ void rb_erase_cached(struct rb_node *node, struct rb_root_cached *root)
  */
 
 void __rb_insert_augmented(struct rb_node *node, struct rb_root *root,
-			   bool newleft, struct rb_node **leftmost,
 	void (*augment_rotate)(struct rb_node *old, struct rb_node *new))
 {
-	__rb_insert(node, root, newleft, leftmost, augment_rotate);
+	__rb_insert(node, root, augment_rotate);
 }
+EXPORT_SYMBOL(__rb_insert_augmented);
 
 /*
  * This function returns the first node (in sort order) of the tree.
  */
-struct rb_node * __attribute__ ((pure))
+struct rb_node * __attribute__((pure))
 rb_first(const struct rb_root *root)
 {
 	struct rb_node	*n;
@@ -512,8 +488,9 @@ rb_first(const struct rb_root *root)
 		n = n->rb_left;
 	return n;
 }
+EXPORT_SYMBOL(rb_first);
 
-struct rb_node * __attribute__ ((pure))
+struct rb_node * __attribute__((pure))
 rb_last(const struct rb_root *root)
 {
 	struct rb_node	*n;
@@ -525,8 +502,9 @@ rb_last(const struct rb_root *root)
 		n = n->rb_right;
 	return n;
 }
+EXPORT_SYMBOL(rb_last);
 
-struct rb_node * __attribute__ ((pure))
+struct rb_node * __attribute__((pure))
 rb_next(const struct rb_node *node)
 {
 	struct rb_node *parent;
@@ -542,7 +520,7 @@ rb_next(const struct rb_node *node)
 	if (node->rb_right) {
 		node1 = node->rb_right;
 		while (node1->rb_left)
-			node1=node1->rb_left;
+			node1 = node1->rb_left;
 		return node1;
 	}
 
@@ -558,9 +536,9 @@ rb_next(const struct rb_node *node)
 
 	return parent;
 }
+EXPORT_SYMBOL(rb_next);
 
-struct rb_node * __attribute__ ((pure))
-rb_prev(const struct rb_node *node)
+struct rb_node *rb_prev(const struct rb_node *node)
 {
 	struct rb_node *parent;
 	struct rb_node *node1;
@@ -575,7 +553,7 @@ rb_prev(const struct rb_node *node)
 	if (node->rb_left) {
 		node1 = node->rb_left;
 		while (node1->rb_right)
-			node1=node1->rb_right;
+			node1 = node1->rb_right;
 		return node1;
 	}
 
@@ -588,6 +566,7 @@ rb_prev(const struct rb_node *node)
 
 	return parent;
 }
+EXPORT_SYMBOL(rb_prev);
 
 void rb_replace_node(struct rb_node *victim, struct rb_node *new,
 		     struct rb_root *root)
@@ -604,15 +583,7 @@ void rb_replace_node(struct rb_node *victim, struct rb_node *new,
 		rb_set_parent(victim->rb_right, new);
 	__rb_change_child(victim, new, parent, root);
 }
-
-void rb_replace_node_cached(struct rb_node *victim, struct rb_node *new,
-			    struct rb_root_cached *root)
-{
-	rb_replace_node(victim, new, &root->rb_root);
-
-	if (root->rb_leftmost == victim)
-		root->rb_leftmost = new;
-}
+EXPORT_SYMBOL(rb_replace_node);
 
 #ifdef _INCLUDE_UNUSED_CODE_
 void rb_replace_node_rcu(struct rb_node *victim, struct rb_node *new,
@@ -635,9 +606,10 @@ void rb_replace_node_rcu(struct rb_node *victim, struct rb_node *new,
 	 */
 	__rb_change_child_rcu(victim, new, parent, root);
 }
+EXPORT_SYMBOL(rb_replace_node_rcu);
 #endif
 
-static struct rb_node * __attribute__ ((pure))
+static struct rb_node * __attribute__((pure))
 rb_left_deepest_node(struct rb_node *node)
 {
 	for (;;) {
@@ -646,11 +618,11 @@ rb_left_deepest_node(struct rb_node *node)
 		else if (node->rb_right)
 			node = node->rb_right;
 		else
-			return node;
+			return (struct rb_node *)node;
 	}
 }
 
-struct rb_node * __attribute__ ((pure))
+struct rb_node * __attribute__((pure))
 rb_next_postorder(const struct rb_node *node)
 {
 	struct rb_node *parent;
@@ -666,14 +638,16 @@ rb_next_postorder(const struct rb_node *node)
 	} else
 		/* Otherwise we are the parent's right node, and the parent
 		 * should be next */
-		return parent;
+		return (struct rb_node *)parent;
 }
+EXPORT_SYMBOL(rb_next_postorder);
 
-struct rb_node * __attribute__ ((pure))
-rb_first_postorder(const struct rb_root *root)
+struct rb_node * __attribute__((pure))
+rb_first_postorder(struct rb_root *root)
 {
 	if (!root->rb_node)
 		return NULL;
 
 	return rb_left_deepest_node(root->rb_node);
 }
+EXPORT_SYMBOL(rb_first_postorder);
