@@ -82,10 +82,6 @@
 #include "rttables.h"
 
 
-#if !defined IPROUTE_USR_DIR && !defined IPROUTE_ETC_DIR
-#define IPROUTE_ETC_DIR "/etc/iproute2"
-#endif
-
 #define RT_TABLES_FILE	"rt_tables"
 #define	RT_DSFIELD_FILE "rt_dsfield"
 #define	RT_REALMS_FILE	"rt_realms"
@@ -193,6 +189,9 @@ static LIST_HEAD_INITIALIZE(rt_scopes);
 static char ret_buf[11];	/* uint32_t in decimal */
 
 static dir_state_t dir_state = DIRS_NOT_CHECKED;
+static const char **iproute_etc_dir;
+static const char **iproute_usr_dir;
+
 
 static void
 free_rt_entry(rt_entry_t *rte)
@@ -255,7 +254,7 @@ read_file(const char *file_name, list_head_t *l, uint32_t max)
 	char *endptr;
 	size_t len;
 
-	fp = fopen(file_name, "r");
+	fp = fopen(file_name, "re");
 	if (!fp)
 		return;
 
@@ -370,9 +369,7 @@ static void
 initialise_list(list_head_t *l, const char *file_name, const rt_entry_t *default_list, uint32_t max)
 {
 	char *path;
-#ifdef IPROUTE_USR_DIR
 	char *etc_path;
-#endif
 	struct stat statbuf;
 	struct dirent *ent;
 	DIR *dir;
@@ -381,12 +378,9 @@ initialise_list(list_head_t *l, const char *file_name, const rt_entry_t *default
 		return;
 
 	if (dir_state == DIRS_NOT_CHECKED) {
-#ifdef IPROUTE_USR_DIR
-		if (!stat(IPROUTE_USR_DIR, &statbuf) && (statbuf.st_mode & S_IFMT) == S_IFDIR)
+		if (*iproute_usr_dir && !stat(*iproute_usr_dir, &statbuf) && (statbuf.st_mode & S_IFMT) == S_IFDIR)
 			dir_state = DIRS_EXIST;
-		else
-#endif
-		if (!stat(IPROUTE_ETC_DIR, &statbuf) && (statbuf.st_mode & S_IFMT) == S_IFDIR)
+		else if (!stat(*iproute_etc_dir, &statbuf) && (statbuf.st_mode & S_IFMT) == S_IFDIR)
 			dir_state = DIRS_EXIST;
 		else
 			dir_state = DIRS_DONT_EXIST;
@@ -394,55 +388,52 @@ initialise_list(list_head_t *l, const char *file_name, const rt_entry_t *default
 
 	if (dir_state == DIRS_EXIST) {
 		path = MALLOC(PATH_MAX);
-#ifdef IPROUTE_USR_DIR
 		etc_path = MALLOC(PATH_MAX);
-#endif
 
-		/* The default location is IPROUTE_USR_DIR, but it is overridden
-		 * if the file exists in IPROUTE_USR_DIR. */
-		snprintf(path, PATH_MAX, "%s/%s", IPROUTE_ETC_DIR, file_name);
+		/* The default location is iproute_usr_dir, but it is overridden
+		 * if the file exists in iproute_usr_dir. */
+		snprintf(path, PATH_MAX, "%s/%s", *iproute_etc_dir, file_name);
 		if (!stat(path, &statbuf) && (statbuf.st_mode & S_IFMT) == S_IFREG)
 			read_file(path, l, max);
-#ifdef IPROUTE_USR_DIR
-		else {
-			snprintf(path, PATH_MAX, "%s/%s", IPROUTE_USR_DIR, file_name);
+		else if (*iproute_usr_dir) {
+			snprintf(path, PATH_MAX, "%s/%s", *iproute_usr_dir, file_name);
 			if (!stat(path, &statbuf) && (statbuf.st_mode & S_IFMT) == S_IFREG)
 				read_file(path, l, max);
 		}
-#endif
 
 		/* iproute2 uses subdirectories for rt_protos, rt_addrprotos, rt_tables
 		 * (and protodown_reasons) as at v6.11.
 		 * To futureproof our code, we will read subdirectories for all files,
 		 * in case iproute2 introduces support for them in the future.
-		 * We need to check all files ending .conf under IPROUTE_USR_DIR and read
-		 * them unless the matching file exists under IPROUTE_ETC_DIR. We then read
-		 * all relevant files under IPROUTE_ETC_DIR. */
-#ifdef IPROUTE_USR_DIR
-		snprintf(path, PATH_MAX, "%s/%s.d", IPROUTE_USR_DIR, file_name);
-		if ((dir = opendir(path))) {
-			while ((ent = readdir(dir))) {
-				if (!wanted_file(path, IPROUTE_USR_DIR, file_name, ent->d_name))
-					continue;
+		 * We need to check all files ending .conf under *iproute_usr_dir
+		 * and read * them unless the matching file exists under
+		 * *iproute_etc_dir. We then read all relevant files under
+		 * *iproute_etc_dir. */
+		if (*iproute_usr_dir) {
+			snprintf(path, PATH_MAX, "%s/%s.d", *iproute_usr_dir, file_name);
+			if ((dir = opendir(path))) {
+				while ((ent = readdir(dir))) {
+					if (!wanted_file(path, *iproute_usr_dir, file_name, ent->d_name))
+						continue;
 
-				/* Check if the file exists in IPROUTE_ETC_DIR. We just check if there is a matching
-				 * entry, and don't care what type the entry is */
-				snprintf(etc_path, PATH_MAX, "%s/%s.d/%s", IPROUTE_ETC_DIR, file_name, ent->d_name);
-				if (!stat(etc_path, &statbuf))
-					continue;
+					/* Check if the file exists in *iproute_etc_dir. We just check
+					 * if there is a matching entry, and don't care what type the entry is */
+					snprintf(etc_path, PATH_MAX, "%s/%s.d/%s", *iproute_etc_dir, file_name, ent->d_name);
+					if (!stat(etc_path, &statbuf))
+						continue;
 
-				read_file(path, l, max);
+					read_file(path, l, max);
+				}
+
+				closedir(dir);
 			}
-
-			closedir(dir);
 		}
-#endif
 
-		/* Now read the entries in the IPROUTE_ETC_DIR subdirectory */
-		snprintf(path, PATH_MAX, "%s/%s.d", IPROUTE_ETC_DIR, file_name);
+		/* Now read the entries in the *iproute_etc_dir subdirectory */
+		snprintf(path, PATH_MAX, "%s/%s.d", *iproute_etc_dir, file_name);
 		if ((dir = opendir(path))) {
 			while ((ent = readdir(dir))) {
-				if (!wanted_file(path, IPROUTE_ETC_DIR, file_name, ent->d_name))
+				if (!wanted_file(path, *iproute_etc_dir, file_name, ent->d_name))
 					continue;
 
 				read_file(path, l, max);
@@ -452,9 +443,7 @@ initialise_list(list_head_t *l, const char *file_name, const rt_entry_t *default
 		}
 
 		FREE_PTR(path);
-#ifdef IPROUTE_USR_DIR
 		FREE_PTR(etc_path);
-#endif
 	}
 
 	if (default_list)
@@ -623,9 +612,10 @@ write_addrproto_config(const char *name, uint32_t val)
 	char *v, *e;
 	int ver_maj, ver_min, ver_rel;
 	char *res;
-	const char *path, *dir = NULL;
-	bool file_exists = false;
+	char *path;
 	struct stat statbuf;
+	size_t len;
+	mode_t old_umask;
 
 	fp = popen("ip -V 2>&1", "re");
 	res = fgets(buf, sizeof(buf), fp);
@@ -657,50 +647,57 @@ write_addrproto_config(const char *name, uint32_t val)
 	if (sscanf(v, "%d.%d.%d", &ver_maj, &ver_min, &ver_rel) != 3)
 		return;
 
-	if (ver_maj >= 7 || (ver_maj == 6 && ver_min >= 13)) {
-		dir = IPROUTE_ETC_DIR "/" RT_ADDRPROTOS_FILE ".d";
-		path = IPROUTE_ETC_DIR "/" RT_ADDRPROTOS_FILE ".d/keepalived.conf" ;
-	} else if (ver_maj == 6 && ver_min >= 3)
-		path = IPROUTE_ETC_DIR "/" RT_ADDRPROTOS_FILE;
-	else
+	if (ver_maj < 6 || (ver_maj == 6 && ver_min < 3))
 		return;
 
-	/* If IPROUTE_ETC_DIR doesn't exist, create it */
-	if (stat(IPROUTE_ETC_DIR, &statbuf)) {
-#ifdef IPROUTE_USR_DIR
-		if (stat(IPROUTE_USR_DIR, &statbuf))
-#endif
-		{
+	/* If *iproute_etc_dir doesn't exist, create it */
+	if (stat(*iproute_etc_dir, &statbuf)) {
+		if (*iproute_usr_dir && stat(*iproute_usr_dir, &statbuf)) {
 			/* Use sensible defaults for directory permission */
 			statbuf.st_mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
 		}
 
 		/* Create directory */
-		if (mkdir(IPROUTE_ETC_DIR, statbuf.st_mode & ~S_IFMT)) {
-			log_message(LOG_INFO, "Unable to create directory " IPROUTE_ETC_DIR " for rt_addrproto keepalived");
+		if (mkdir(*iproute_etc_dir, statbuf.st_mode & ~S_IFMT)) {
+			log_message(LOG_INFO, "Unable to create directory %s for rt_addrproto keepalived",
+				    *iproute_etc_dir);
 			return;
 		}
 	}
 
-	if (dir) {
-		if (!mkdir(dir, statbuf.st_mode & ~S_IFMT))	// This may fail if the directory already exists
-			chmod(dir, statbuf.st_mode & ~S_IFMT);
-		else if (errno == EEXIST)
-			file_exists = !stat(path, &statbuf);
+	if (ver_maj >= 7 || (ver_maj == 6 && ver_min >= 13)) {
+		len = strlen(*iproute_etc_dir) + 1 + strlen(RT_ADDRPROTOS_FILE) + strlen(".d") + 1 + strlen("keepalived.conf") + 1;
+		if (len > PATH_MAX) {
+			log_message(LOG_INFO, "%s/%s.d/%s exceeds PATH_MAX (%d) characters", *iproute_etc_dir, RT_ADDRPROTOS_FILE, "keepalived.conf", PATH_MAX);
+			return;
+		}
+
+		path = MALLOC(len);
+		snprintf(path, len, "%s/%s.d/", *iproute_etc_dir, RT_ADDRPROTOS_FILE);
+
+		if (!mkdir(path, statbuf.st_mode & ~S_IFMT))	// This may fail if the directory already exists
+			chmod(path, statbuf.st_mode & ~S_IFMT);
+
+		strncat(path, "keepalived.conf", len);
 	} else {
-		/* Check if rt_addrprotos file exists */
-		file_exists = !stat(path, &statbuf);
+		len = strlen(*iproute_etc_dir) + 1 + strlen(RT_ADDRPROTOS_FILE) + 1;
+		if (len > PATH_MAX) {
+			log_message(LOG_INFO, "%s/%s exceeds PATH_MAX (%d) characters", *iproute_etc_dir, RT_ADDRPROTOS_FILE, PATH_MAX);
+			return;
+		}
+		path = MALLOC(len);
+		snprintf(path, len, "%s/%s", *iproute_etc_dir, RT_ADDRPROTOS_FILE);
 	}
 
-	if (!(fp = fopen(path, "a")))
-		return;
+	old_umask = umask(S_IXUSR | S_IWGRP | S_IXGRP | S_IWOTH | S_IXOTH);
+	if ((fp = fopen(path, "ae"))) {
+		fprintf(fp, "%u\t%s\t# added by keepalived\n", val, name);
 
-	if (!file_exists)
-		chmod(path, statbuf.st_mode & ~S_IFMT & ~(S_IXUSR | S_IXGRP | S_IXOTH));
+		fclose(fp);
+	}
+	umask(old_umask);
 
-	fprintf(fp, "%u\t%s\t# added by keepalived\n", val, name);
-
-	fclose(fp);
+	FREE_PTR(path);
 }
 
 bool
@@ -757,3 +754,10 @@ find_rttables_addrproto(const char *name, uint8_t *id)
 	return true;
 }
 #endif
+
+void
+set_iproute_dirs(const char **etc_dir, const char **usr_dir)
+{
+	iproute_etc_dir = etc_dir;
+	iproute_usr_dir = usr_dir;
+}
