@@ -33,6 +33,7 @@
 
 #include "systemd.h"
 #include "logger.h"
+#include "process.h"
 
 static bool parent_is_systemd;
 
@@ -47,7 +48,7 @@ check_parent_systemd(void)
 	if (getppid() != 1)
 		return false;
 
-	if ((fd = open("/proc/1/stat", O_RDONLY)) == -1)
+	if ((fd = open("/proc/1/stat", O_RDONLY | O_CLOEXEC)) == -1)
 		return false;
 
 	len = read(fd, stat_buf, sizeof(stat_buf) - 1);
@@ -74,7 +75,7 @@ void
 systemd_notify_running(void)
 {
 	if (parent_is_systemd)
-		sd_notifyf(0, "READY=1\nMAINPID=%lu", (unsigned long)getpid());
+		sd_notifyf(0, "READY=1\nMAINPID=%lu", (unsigned long)our_pid);
 }
 
 void
