@@ -47,12 +47,24 @@
 #include <sys/epoll.h>
 #include <sys/inotify.h>
 #endif
+#if defined USE_CLOSE_RANGE_SYSCALL
+#include <sys/syscall.h>
+#endif
+#if !defined HAVE_CLOSE_RANGE || !HAVE_DECL_CLOSE_RANGE_CLOEXEC
+#include <dirent.h>
+#include <stdlib.h>
+#include <ctype.h>
+#endif
 
 #ifdef _WITH_STACKTRACE_
 #include <sys/stat.h>
 #include <execinfo.h>
 #include <memory.h>
 #endif
+#ifdef DO_STACKTRACE
+#include <inttypes.h>
+#include <unistd.h>
+#endif
 #ifdef _HAVE_LIBKMOD_
 #include <libkmod.h>
 #endif
@@ -68,6 +80,10 @@
 #include "process.h"
 #include "timer.h"
 
+#if defined USE_CLOSE_RANGE_SYSCALL && !defined SYS_close_range
+#define SYS_close_range __NR_close_range
+#endif
+
 /* global vars */
 unsigned long debug = 0;
 mode_t umask_val = S_IXUSR | S_IRWXG | S_IRWXO;
@@ -77,6 +93,13 @@ const char *tmp_dir;
 bool do_eintr_debug;
 #endif
 
+#ifdef DO_STACKSIZE
+#define STACK_UNUSED 0xdeadbeeffeedcafe
+#define STACKSIZE_DEBUG 0
+
+static void *orig_stack_base;
+#endif
+
 /* Display a buffer into a HEXA formated output */
 void
 dump_buffer(const char *buff, size_t count, FILE* fp, int indent)
@@ -161,7 +184,7 @@ write_stacktrace(const char *file_name, const char *str)
 
 	nptrs = backtrace(buffer, 100);
 	if (file_name) {
-		fd = open(file_name, O_WRONLY | O_APPEND | O_CREAT | O_NOFOLLOW, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
+		fd = open(file_name, O_WRONLY | O_APPEND | O_CREAT | O_NOFOLLOW | O_CLOEXEC, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
 		if (str)
 			dprintf(fd, "%s\n", str);
 		backtrace_symbols_fd(buffer, nptrs, fd);
@@ -191,7 +214,7 @@ write_stacktrace(const char *file_name, const char *str)
 	else if (file_name[0] != '/')
 		tmp_filename = make_tmp_filename(file_name);
 	cmd = MALLOC(6 + 1 + PID_MAX_DIGITS + 1 + 2 + ( tmp_filename ? strlen(tmp_filename) : strlen(file_name)) + 1);
-	sprintf(cmd, "gstack %d >>%s", getpid(), tmp_filename ? tmp_filename : file_name);
+	sprintf(cmd, "gstack %d >>%s", our_pid, tmp_filename ? tmp_filename : file_name);
 
 	i = system(cmd);	/* We don't care about return value but gcc thinks we should */
 
@@ -200,6 +223,88 @@ write_stacktrace(const char *file_name, const char *str)
 }
 #endif
 
+#ifdef DO_STACKSIZE
+RELAX_STACK_PROTECTOR_START
+int
+get_stacksize(bool end)
+{
+	/* We use a struct for all local variables so that we
+	 * know the address of the lowest variable on the stack */
+	struct {
+		void *stack_base, *stack_top;
+		uintptr_t write_end;
+		uint64_t *p;
+		FILE *fp;
+		uintptr_t aligned_base;
+		unsigned num_ent;
+		unsigned i;
+		uint64_t *base;
+		int page_size;
+		char buf[257];
+	} s;
+
+	if (!(s.fp = fopen("/proc/self/maps", "re")))
+		return -1;
+
+	while (fgets(s.buf, sizeof(s.buf), s.fp)) {
+		if (!strstr(s.buf, "[stack]\n"))
+			continue;
+
+		sscanf(s.buf, "%p-%p ", &s.stack_base, &s.stack_top);
+		break;
+	}
+	fclose(s.fp);
+
+#if STACKSIZE_DEBUG
+	log_message(LOG_INFO, "stack from %p to %p, stack now ~= %p", s.stack_base, s.stack_top, &s);
+#endif
+
+	s.write_end = (uintptr_t)&s & ~(sizeof(uint64_t) - 1);
+	s.page_size = sysconf(_SC_PAGESIZE);
+
+	if (!end) {
+		s.aligned_base = (uintptr_t)s.stack_base;
+		s.aligned_base &= ~(s.page_size - 1);
+		s.num_ent = ((char *)s.write_end - (char *)s.aligned_base) / sizeof(uint64_t) - 1;
+		s.num_ent -= s.page_size / sizeof(uint64_t);
+RELAX_ALLOCA_START
+		s.base = alloca(s.num_ent * sizeof(uint64_t));
+RELAX_ALLOCA_END
+#if STACKSIZE_DEBUG
+		log_message(LOG_INFO, "alloca() gave us %p, &s = %p", s.base, &s);
+#endif
+		for (s.p = s.base, s.i = 0; s.i < s.num_ent; s.i++, s.p++)
+			*s.p = STACK_UNUSED;
+
+		orig_stack_base = s.stack_base;
+	} else if (s.stack_base != orig_stack_base)
+#if STACKSIZE_DEBUG
+		log_message(LOG_INFO, "Stack base changed from %1$p to %2$p, used > 0x%3$lx (%3$lu) bytes", orig_stack_base, s.stack_base,
+			    (unsigned long)((char *)s.stack_top - (char *)orig_stack_base));
+#else
+		log_message(LOG_INFO, "Stack used > 0x%1$lx (%1$lu) bytes",
+			    (unsigned long)((char *)s.stack_top - (char *)orig_stack_base));
+#endif
+	else {
+		for (s.p = (uint64_t *)((char *)s.stack_base + s.page_size); s.p != s.stack_top; s.p++) {
+			if (*s.p != STACK_UNUSED)
+				break;
+		}
+
+#if STACKSIZE_DEBUG
+		log_message(LOG_INFO, "Lowest stack use at %1$p, value %2$" PRIx64 ", used 0x%3$lx (%3$lu) bytes", s.p, *s.p,
+			    (unsigned long)((char *)s.stack_top - (char *)s.p));
+#else
+		log_message(LOG_INFO, "Stack used 0x%1$lx (%1$lu) bytes",
+			    (unsigned long)((char *)s.stack_top - (char *)s.p));
+#endif
+	}
+
+	return 0;
+}
+RELAX_STACK_PROTECTOR_END
+#endif
+
 const char *
 make_file_name(const char *name, const char *prog, const char *namespace, const char *instance)
 {
@@ -314,8 +419,8 @@ run_perf(const char *process, const char *network_namespace, const char *instanc
 		struct inotify_event *ie = PTR_CAST(struct inotify_event, buf);
 		struct epoll_event ee = { .events = EPOLLIN, .data.fd = in };
 
-		if ((ep = epoll_create(1)) == -1) {
-			log_message(LOG_INFO, "perf epoll_create failed errno %d - %m", errno);
+		if ((ep = epoll_create1(EPOLL_CLOEXEC)) == -1) {
+			log_message(LOG_INFO, "perf epoll_create1 failed errno %d - %m", errno);
 			break;
 		}
 
@@ -1042,7 +1147,7 @@ fopen_safe(const char *path, const char *mode)
 
 	if ((mode[0] != 'a' && mode[0] != 'w') ||
 	    (mode[1] &&
-	     (mode[1] != '+' || mode[2]))) {
+	     ((mode[1] != 'e' && mode[1] != '+') || mode[2]))) {
 		errno = EINVAL;
 		return NULL;
 	}
@@ -1076,7 +1181,7 @@ fopen_safe(const char *path, const char *mode)
 		errno = EINVAL;
 		return NULL;
 #else
-		flags = O_NOFOLLOW | O_CREAT | O_CLOEXEC | O_APPEND;
+		flags |= O_APPEND;
 
 		if (mode[1])
 			flags |= O_RDWR;
@@ -1125,7 +1230,7 @@ fopen_safe(const char *path, const char *mode)
 		}
 	}
 
-	file = fdopen (fd, "w");
+	file = fdopen(fd, mode);
 	if (!file) {
 		sav_errno = errno;
 		log_message(LOG_INFO, "fdopen(\"%s\") failed - errno %d (%m)", path, errno);
@@ -1379,6 +1484,55 @@ make_tmp_filename(const char *file_name)
 	return path;
 }
 
+#if defined USE_CLOSE_RANGE_SYSCALL
+int
+close_range(unsigned first, unsigned last, int flags)
+{
+	int ret;
+
+	ret = syscall(SYS_close_range, first, last, flags);
+
+	return ret;
+}
+#endif
+
+#if !defined HAVE_CLOSE_RANGE || !HAVE_DECL_CLOSE_RANGE_CLOEXEC
+unsigned
+get_open_fds(uint64_t *fds, unsigned num_ent)
+{
+	DIR *dir = opendir("/proc/self/fd");
+	struct dirent *ent;
+	unsigned long fd_num;
+	unsigned i;
+	unsigned max_fd = 0;
+	struct stat stbuf;
+
+	for (i = 0; i < num_ent; i++)
+		fds[i] = 0;
+	
+	while ((ent = readdir(dir))) {
+		// Allow for . and ..
+		if (!isdigit(ent->d_name[0]))
+			continue;
+
+		fd_num = strtoul(ent->d_name, NULL, 10);
+
+		/* Make sure it isn't a directory - i.e. the fd returned by opendir() */
+		if (fstat(fd_num, &stbuf) || S_ISDIR(stbuf.st_mode))
+			continue;
+
+		fds[fd_num / 64] |= 1UL << (fd_num % 64) ;
+
+		if (fd_num > max_fd)
+			max_fd = fd_num;
+	}
+
+	closedir(dir);
+
+	return max_fd;
+}
+#endif
+
 void
 log_stopping(void)
 {
